@@ -1,7 +1,4 @@
-## Main function: BDMCMC algorithm for selecting the best graphs 
-# bdgraph with missing data option in copula
-################################################################################
-# in option "copulaNA", I should check ", PACKAGE = BDgraph)"
+## Main function: BDMCMC algorithm for graphical models 
 ################################################################################
 bdgraph = function( data, n = NULL, method = "ggm", iter = 5000, 
 					burnin = iter / 2, b = 3, D = NULL, Gstart = "empty" )
@@ -11,56 +8,58 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 	
 	if ( class(data) == "simulate" ) data <- data $ data
 
-	if ( is.matrix(data) == FALSE & is.data.frame(data) == FALSE ) stop( "Data should be a matrix or dataframe" )
+	if ( !is.matrix(data) & !is.data.frame(data) ) stop( "Data should be a matrix or dataframe" )
 	if ( is.data.frame(data) ) data <- data.matrix(data)
 	if ( iter <= burnin )   stop( "Number of iteration must be more than number of burn-in" )
+	gcgm_NA = 0
 	if ( any( is.na(data) ) ) 
 	{
-		if ( method == "ggm" ) 
-		{
-			stop( "ggm method does not deal with missing value. You could choose method = cggm" )
-		} else {
-			method = "cggm-NA"
-		}
+		if( method == "ggm" ) stop( "ggm method does not deal with missing value. You could choose method = gcgm" )	
+		gcgm_NA = 1
 	}
 		
 	dimd <- dim(data)
 	p    <- dimd[2]
-	n    <- dimd[1]
+	if ( is.null(n) ) n <- dimd[1]
 
-	if ( method == "cggm" | method == "cggm-NA" | method == "cggm-dmh" | method == "cggm-dmh-NA" )
+	if ( method == "gcgm" )
 	{
-		Z              <- qnorm( apply( data, 2, rank, ties.method = "random" ) / (n + 1) )
-		Zfill          <- matrix( rnorm( n * p ), n, p )   # for missing values
-		Z[is.na(data)] <- Zfill[is.na(data)]               # for missing values
-		Z              <- t( ( t(Z) - apply( Z, 2, mean ) ) / apply( Z, 2, sd ) )
-		S              <- t(Z) %*% Z
-		# ?? I should check
 		R <- 0 * data
-		for ( j in 1:p ) 
-			R[,j] = match( data[ , j], sort( unique( data[ , j] ) ) ) 
-		R[ is.na(R) ] = 0     # dealing with missing values
-	} else {
+		for ( j in 1:p ) R[,j] = match( data[ , j], sort( unique( data[ , j] ) ) ) 
+		R[ is.na(R) ] = 0     # dealing with missing values	
+
+		# copula for continuous non-Gaussian data
+		if( gcgm_NA != 1 && min( apply( R, 2, max ) ) > ( n - 5 * n / 100 ) )
+		{
+			# copula transfer 
+			data = qnorm( apply( data, 2, rank ) / ( n + 1 ) )
+#~ 			data = data / sd( data[ , 1] )
+		
+			method = "ggm"
+		}
+		else 
+		{	# for non-Gaussian data
+			Z              <- qnorm( apply( data, 2, rank, ties.method = "random" ) / (n + 1) )
+			Zfill          <- matrix( rnorm( n * p ), n, p )   # for missing values
+			Z[is.na(data)] <- Zfill[is.na(data)]               # for missing values
+			Z              <- t( ( t(Z) - apply( Z, 2, mean ) ) / apply( Z, 2, sd ) )
+			S              <- t(Z) %*% Z
+		}
+	} 
+	
+	if( method == "ggm" ) 
+	{
 		if ( isSymmetric(data) )
 		{
 			if ( is.null(n) ) stop( "Please specify the number of observations 'n'" )
-			cat( "The input is identified as the covriance matrix. \n" )
+			cat( "Input is identified as the covriance matrix. \n" )
 			S <- data
 		} else {
-			S <- t(data) %*% data
+ 			S <- t(data) %*% data
 		}
 	}
 
-	if ( is.null(D) )
-	{ 
-		D  <- diag(p)
-		Ti <- D
-	} else {
-		Ti <- chol( solve(D) )
-		if ( method == "ggm" )     method == "ggm-approx"
-		if ( method == "cggm" )    method == "cggm-dmh"
-		if ( method == "cggm-NA" ) method == "cggm-dmh-NA"
-	}
+	if ( is.null(D) ) D = diag(p)
 
 	bstar <- b + n
 	Ds    <- D + S
@@ -82,9 +81,8 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 	if ( class(Gstart) == "character" && Gstart == "empty"  )
 	{
 		G = 0 * S
+		K = matrix( 0, p, p )
 		
-		K <- matrix( 0, p, p )
-		# rgwish ( integer G[], double Ti[], double K[], int *b, int *p )
 		result = .C( "rgwish", as.integer(G), as.double(Ts), K = as.double(K), as.integer(bstar), 
 					  as.integer(p), PACKAGE = "BDgraph" )
 		K = matrix ( result $ K, p, p ) 
@@ -94,9 +92,8 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 	{
 		G       = matrix(1, p, p)
 		diag(G) = 0
-	
-		K = matrix( 0, p, p)
-		# rwish ( double Ti[], double K[], int *p, int *b )
+		K       = matrix( 0, p, p)
+
 		result = .C( "rwish", as.double(Ts), K = as.double(K), as.integer(bstar), as.integer(p), PACKAGE = "BDgraph" )
 		K = matrix ( result $ K, p, p ) 
 	}	
@@ -105,9 +102,8 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 	{
 		G       = Gstart
 		diag(G) = 0
+		K       = matrix( 0, p, p )
 		
-		K <- matrix( 0, p, p )
-		# rgwish ( integer G[], double Ti[], double K[], int *b, int *p )
 		result = .C( "rgwish", as.integer(G), as.double(Ts), K = as.double(K), as.integer(bstar), 
 					  as.integer(p), PACKAGE = "BDgraph" )
 		K = matrix ( result $ K, p, p ) 	
@@ -115,9 +111,8 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 		
 	allGraphs    <- c( rep ( "a", iter ) ) # vector of numbers like "10100"
 	allWeights   <- c( rep ( 0, iter ) )   # waiting time for every state		
-
-	sampleGraphs <- c( rep ( "a", iter ) ) # vector of numbers like "10100" 
-	graphWeights <- c( rep ( 0, iter ) )   # waiting time for every state
+	sampleGraphs <- allGraphs              # vector of numbers like "10100" 
+	graphWeights <- allWeights             # waiting time for every state
 	
 	sizeSampleG = 0
 
@@ -128,15 +123,9 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 
 	mes <- paste( c(iter," iteration is started.                    " ), collapse = "" )
 	cat( mes, "\r" )
-	#flush.console()
 
 	if ( method == "ggm" )
 	{
-# bdmcmcExact( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double Ds[] )   
 		result = .C( "bdmcmcExact", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ts), as.double(K), as.integer(p), 
 					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
 				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
@@ -145,46 +134,10 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 				    , PACKAGE = "BDgraph" )
 	}
 	
-	if ( method == "ggm-dmh" )
-	{	
-# bdmcmcDmh( int *iter, int *burnin, int G[], double Ti[], double Ts[], double K[], int *p, 
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double D[], double Ds[] )
-		result = .C( "bdmcmcDmh", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ti), as.double(Ts), as.double(K), as.integer(p), 
-					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
-				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
-				    lastGraph = as.integer(lastGraph), lastK = as.double(lastK),
-				    as.integer(b), as.integer(bstar), as.double(D), as.double(Ds)
-				    , PACKAGE = "BDgraph" )
-	}
-
-	if ( method == "ggm-approx" )
+	if ( method == "gcgm" )
 	{
-# bdmcmcApprox( int *iter, int *burnin, int G[], double Ti[], double Ts[], double K[], int *p, 
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double Ds[] )
-		result = .C( "bdmcmcApprox", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ti), as.double(Ts), as.double(K), as.integer(p), 
-					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
-				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
-				    lastGraph = as.integer(lastGraph), lastK = as.double(lastK),
-				    as.integer(b), as.integer(bstar), as.double(Ds)
-				    , PACKAGE = "BDgraph" )
-	}
-
-	if ( method == "cggm" )
-	{
-# bdmcmcCopula( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
-#			 double Z[], int R[], int *n,
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double D[], double Ds[] )
 		result = .C( "bdmcmcCopula", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ts), as.double(K), as.integer(p),
-		            as.double(Z), as.integer(R), as.integer(n),
+		            as.double(Z), as.integer(R), as.integer(n), as.integer(gcgm_NA),
 					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
 				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
 				    lastGraph = as.integer(lastGraph), lastK = as.double(lastK),
@@ -192,57 +145,6 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 				    , PACKAGE = "BDgraph" )
 	}
 
-	if ( method == "cggm-NA" )
-	{
-# bdmcmcCopulaNA( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
-#			 double Z[], int R[], int *n,
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double D[], double Ds[] )
-		result = .C( "bdmcmcCopulaNA", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ts), as.double(K), as.integer(p),
-		            as.double(Z), as.integer(R), as.integer(n),
-					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
-				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
-				    lastGraph = as.integer(lastGraph), lastK = as.double(lastK),
-				    as.integer(b), as.integer(bstar), as.double(D), as.double(Ds) 
-				    , PACKAGE = "BDgraph" )
-	}
-
-	if ( method == "cggm-dmh" )
-	{
-# bdmcmcCopulaDmh( int *iter, int *burnin, int G[], double Ti[], double Ts[], double K[], int *p, 
-#			 double Z[], int R[], int *n,
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double D[], double Ds[] )
-		result = .C( "bdmcmcCopulaDmh", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ti), as.double(Ts), as.double(K), as.integer(p),
-		            as.double(Z), as.integer(R), as.integer(n),
-					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
-				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
-				    lastGraph = as.integer(lastGraph), lastK = as.double(lastK),
-				    as.integer(b), as.integer(bstar), as.double(D), as.double(Ds)
-				    , PACKAGE = "BDgraph" )
-	}
-
-	if ( method == "cggm-dmh-NA" )
-	{
-# bdmcmcCopulaDmhNA( int *iter, int *burnin, int G[], double Ti[], double Ts[], double K[], int *p, 
-#			 double Z[], int R[], int *n,
-#			 string allGraphs[], double allWeights[], double Ksum[], 
-#			 string sampleGraphs[], double graphWeights[], int *sizeSampleG,
-#			 int lastGraph[], double lastK[],
-#			 int *b, int *bstar, double D[], double Ds[] )
-		result = .C( "bdmcmcCopulaDmhNA", as.integer(iter), as.integer(burnin), as.integer(G), as.double(Ti), as.double(Ts), as.double(K), as.integer(p),
-		            as.double(Z), as.integer(R), as.integer(n),
-					allGraphs = as.character(allGraphs), allWeights = as.double(allWeights), Ksum = as.double(Ksum), 
-				    sampleGraphs = as.character(sampleGraphs), graphWeights = as.double(graphWeights), sizeSampleG = as.integer(sizeSampleG),
-				    lastGraph = as.integer(lastGraph), lastK = as.double(lastK),
-				    as.integer(b), as.integer(bstar), as.double(D), as.double(Ds) 
-				    , PACKAGE = "BDgraph" )
-	}
-	
 	Ksum         = matrix( result $ Ksum, p, p )
 	allGraphs    = result $ allGraphs
 	allWeights   = result $ allWeights
@@ -261,41 +163,41 @@ bdgraph = function( data, n = NULL, method = "ggm", iter = 5000,
 	class( output ) <- "bdgraph"
 	return( output )   
 }
-     
+      
 # plot for class bdgraph
 plot.bdgraph = function( x, g = 1, layout = layout.circle, ... )
 {
 	list.G  <- x $ sampleGraphs
 	graphWeights <- x $ graphWeights
 	p       <- nrow( x $ lastGraph )
-	prob.G  <- graphWeights / sum(graphWeights)
+	prob.G  <- graphWeights / sum( graphWeights )
 	graphi  <- list()
 	gv      <- c(rep(0, p * (p - 1) / 2))
 
-	if (g == 2) op <- par(mfrow = c(1, 2), pty = "s")
-	if (g > 2 & g < 7)  op <- par(mfrow = c(2, g %% 2 + trunc(g / 2)), pty = "s")
+	if ( g == 2 ) op <- par( mfrow = c( 1, 2 ), pty = "s" )
+	if ( g > 2 & g < 7 )  op <- par( mfrow = c( 2, g %% 2 + trunc( g / 2 ) ), pty = "s" )
 
-	for (i in 1 : g)
+	for ( i in 1 : g )
 	{
 		if (g > 6) dev.new()  
-		gi <- list.G[which(prob.G == sort(prob.G, decreasing = T)[i])]
+		gi <- list.G[ which( prob.G == sort( prob.G, decreasing = T )[i] ) ]
 		gv <- 0 * gv
-		gv[which(unlist(strsplit(as.character(gi), "")) == 1)] <- 1
-		graphi[[i]] <- matrix(0, p, p)
-		graphi[[i]][upper.tri(graphi[[i]])] <- gv
-		colnames(graphi[[i]]) = colnames( x $ lastGraph )
-		rownames(graphi[[i]]) = colnames(graphi[[i]])
-		G    <- graph.adjacency(graphi[[i]], mode = "undirected", diag = FALSE)
-		main <- ifelse (i == 1, "Graph with highest probability", paste(c(i, "th graph"), collapse = ""))
-		plot.igraph(G, layout = layout, main = main, sub = paste(c("Posterior probability = ", 
-		            round(sort(prob.G, decreasing = TRUE)[i], 4)), collapse = ""), ...)	   
+		gv[ which( unlist( strsplit( as.character(gi), "" ) ) == 1 ) ] <- 1
+		graphi[[i]] <- matrix( 0, p, p )
+		graphi[[i]][ upper.tri( graphi[[i]] ) ] <- gv
+		colnames( graphi[[i]] ) = colnames( x $ lastGraph )
+		rownames( graphi[[i]] ) = colnames( graphi[[i]] )
+		G    <- graph.adjacency( graphi[[i]], mode = "undirected", diag = FALSE )
+		main <- ifelse ( i == 1, "Graph with highest probability", paste( c( i, "th graph" ), collapse = "" ) )
+		plot.igraph( G, layout = layout, main = main, sub = paste( c( "Posterior probability = ", 
+		            round( sort( prob.G, decreasing = TRUE )[i], 4 ) ), collapse = "" ), ... )	   
 	}
 	
-	if (g > 1 & g < 7) par(op)
+	if ( g > 1 & g < 7 ) par( op )
 }
    
 # summary of bdgraph output
-summary.bdgraph = function( object, vis = TRUE, layout = layout.circle, ... )
+summary.bdgraph = function( object, vis = TRUE, ... )
 {
 	sampleGraphs <- object $ sampleGraphs
 	graphWeights <- object $ graphWeights
@@ -314,19 +216,19 @@ summary.bdgraph = function( object, vis = TRUE, layout = layout.circle, ... )
 
 	if ( vis )
 	{
-		# plot best graph
+		# plot selected graph (graph with the highest posterior probability)
 		G  <- graph.adjacency( graphi, mode = "undirected", diag = FALSE )
 		 
 		op = par( mfrow = c(2, 2), pty = "s", omi = c(0.3,0.3,0.3,0.3), mai = c(0.3,0.3,0.3,0.3) ) 
 
-		plot.igraph(G, layout = layout, main = "Best graph",
-		  sub = paste( c( "Posterior probability = ", max( prob.G ) ), collapse = "" ), ... )
+		if ( p < 20 ) size = 15 else size = 2
+		plot.igraph(G, layout = layout.circle, main = "Selected graph", sub = paste( c( "Posterior probability = ", max( prob.G ) ), collapse = "" ), vertex.color = "white", vertex.size = size, vertex.label.color = 'black' )
 		
 		# plot posterior distribution of graph
 		plot(x = 1 : length(graphWeights), y = graphWeights / sum(graphWeights), type = "h", main = "Posterior probability of graphs",
 			 ylab = "Pr(graph|data)", xlab = "graph")
 		abline(h = max(graphWeights) / sum(graphWeights), col = "red")
-		text(which(max(graphWeights) == graphWeights), max(graphWeights) / sum(graphWeights), "P(best graph|data)", col = "gray60", adj = c(0, + 1))
+		text(which(max(graphWeights) == graphWeights), max(graphWeights) / sum(graphWeights), "Pr(selected graph|data)", col = "gray60", adj = c(0, + 1))
 		
 		# plot posterior distribution of graph size
 		suma     <- sapply( sampleGraphs, function(x) length( which( unlist( strsplit( as.character(x), "" ) ) == 1 ) ) )
@@ -366,7 +268,7 @@ summary.bdgraph = function( object, vis = TRUE, layout = layout.circle, ... )
 	# estimation for precision matrix 
 	Khat        <- object $ Khat
 
-	return.list <- list(best.graph = Matrix(graphi + t(graphi), sparse = TRUE), phat = Matrix(round(phat, 2), 
+	return.list <- list(selected_graph = Matrix(graphi + t(graphi), sparse = TRUE), phat = Matrix(round(phat, 2), 
 					  sparse = TRUE), Khat = round(Khat, 3))
 					  
 	return(return.list)
@@ -378,7 +280,7 @@ print.bdgraph = function(x, round = 3, Khat = FALSE, phat = FALSE, ...)
 	sampleGraphs <- x $ sampleGraphs
 	graphWeights  <- x $ graphWeights
 	p        <- nrow( x $ lastGraph )
-	# best graph
+	# selected graph
 	prob.G   <- graphWeights / sum(graphWeights)
 	gv       <- c(rep(0, p * (p - 1) / 2))
 	gi       <- sampleGraphs[which(prob.G == max(prob.G))]
@@ -390,27 +292,27 @@ print.bdgraph = function(x, round = 3, Khat = FALSE, phat = FALSE, ...)
 	graphi <- matrix( 0, p, p, dimnames = list( dimlab, dimlab ) )	
 
 	graphi[upper.tri(graphi)] <- gv
-	cat(paste(""), fill = TRUE)
-	cat(paste("Adjacency matrix of best graph"), fill = TRUE)
-	cat(paste(""), fill = TRUE)
+	cat( paste( "" ), fill = TRUE )
+	cat( paste( "Adjacency matrix of selected graph" ), fill = TRUE )
+	cat( paste( "" ), fill = TRUE )
 	printSpMatrix(Matrix(graphi + t(graphi), sparse = TRUE), col.names = TRUE, note.dropping.colnames = FALSE)
 
 	cat( paste( "" ), fill = TRUE )
-	cat( paste( "Size of best graph =", sum(graphi) ), fill = TRUE )
-	cat( paste( "Posterior probability of best graph = ", max(graphWeights) / sum(graphWeights) ), fill = TRUE )  
+	cat( paste( "Size of selected graph =", sum(graphi) ), fill = TRUE )
+	cat( paste( "Posterior probability of selected graph = ", max(graphWeights) / sum(graphWeights) ), fill = TRUE )  
 	cat( paste( ""), fill = TRUE )
 
 	# print for precision matrix
-	if (Khat == TRUE)
+	if ( Khat )
 	{
-		cat(paste(""), fill = TRUE)
-		cat(paste("Estimation of precision matrix"), fill = TRUE)
-		cat(paste(""), fill = TRUE)
-		print(round(x $ Khat, round))
+		cat( paste( "" ), fill = TRUE )
+		cat( paste( "Estimation of precision matrix" ), fill = TRUE )
+		cat( paste( "" ), fill = TRUE )
+		print( round( x $ Khat, round ) )
 	}
 
 	# print for phat
-	if (phat == TRUE)
+	if ( phat )
 	{
 		pvec <- 0 * gv
 		for (i in 1 : length(sampleGraphs))
@@ -421,11 +323,11 @@ print.bdgraph = function(x, round = 3, Khat = FALSE, phat = FALSE, ...)
 
 		phat                  <- 0 * graphi
 		phat[upper.tri(phat)] <- pvec / sum(graphWeights)
-		cat(paste(""), fill = TRUE)
-		cat(paste("Posterior probability of links"), fill = TRUE)
-		cat(paste(""), fill = TRUE)
+		cat( paste( "" ), fill = TRUE )
+		cat( paste( "Posterior probability of links" ), fill = TRUE )
+		cat( paste( "" ), fill = TRUE )
 
-		printSpMatrix(Matrix(round(phat, round), sparse = TRUE), col.names = TRUE, note.dropping.colnames = FALSE)  
+		printSpMatrix( Matrix( round( phat, round ), sparse = TRUE ), col.names = TRUE, note.dropping.colnames = FALSE )  
 	}
 } 
    
